import {
  isArray,
  isBinaryArray,
  isObject,
  isPrimitive,
  isSymbol
} from "./chunk-V5ZQ2SFQ.js";

// src/common/data/object.ts
function objectMap(obj, fn) {
  if (!isObject(obj))
    return {};
  return Object.fromEntries(
    Object.entries(obj).map(([k, v]) => {
      const r = fn(k, v);
      if (isArray(r) && r.length === 2)
        return r;
      return [k, r];
    }).filter((v) => v != null)
  );
}
function objectMergeDisposable(a, b) {
  if (b.dispose && a.dispose) {
    b.dispose.add(a.dispose);
  }
  return Object.assign(a, b);
}
function objectIsEmpty(obj) {
  return Object.keys(obj).length <= 0;
}
function objectPick(obj, ...keys) {
  return Object.fromEntries(keys.filter((key) => key in obj).map((key) => [key, obj[key]]));
}
function objectInclusivePick(obj, ...keys) {
  return Object.fromEntries(keys.map((key) => [key, obj[key]]));
}
function objectOmit(obj, ...keys) {
  return Object.fromEntries(Object.entries(obj).filter(([key]) => !keys.includes(key)));
}
function objectPlain(obj, opt) {
  const {
    maxDepth = 99,
    circleValue,
    maxDepthValue,
    errorTrace = true,
    filter = () => true,
    keepAsIs = () => false,
    transformer = () => void 0
  } = opt ?? {};
  const cycle = [];
  function handleObject(obj2, depth) {
    if (depth > maxDepth)
      return maxDepthValue;
    if (keepAsIs(obj2))
      return obj2;
    if (!isPrimitive(obj2)) {
      if (cycle.includes(obj2))
        return circleValue;
      cycle.push(obj2);
    }
    if (transformer) {
      const result = transformer(obj2);
      if (result !== void 0)
        return result;
    }
    if (isSymbol(obj2))
      return String(obj2);
    if (isPrimitive(obj2))
      return obj2;
    if (obj2 instanceof Date) {
      return {
        __class: "Date",
        value: obj2.toISOString()
      };
    }
    if (obj2 instanceof RegExp) {
      return {
        __class: "RegExp",
        source: obj2.toString()
      };
    }
    if (obj2 instanceof Map)
      obj2 = Object.fromEntries(obj2);
    if (obj2 instanceof Set || isBinaryArray(obj2))
      obj2 = Array.from(obj2);
    if (obj2 instanceof Error) {
      return {
        __class: "Error",
        name: obj2.name,
        message: obj2.message,
        stack: errorTrace ? obj2.stack : void 0,
        cause: obj2.cause ? String(obj2.cause) : void 0
      };
    }
    if (Array.isArray(obj2)) {
      return obj2.filter(filter).map((o) => handleObject(o, depth + 1));
    }
    const objName = obj2?.constructor?.name;
    if (objName && objName !== "Object") {
      const nobj2 = {
        __class: objName
        // __code: obj.toString ? obj.toString() : undefined,
      };
      for (const k in obj2) {
        if (Object.hasOwn(obj2, k) && isPrimitive(obj2[k]))
          nobj2[k] = obj2[k];
      }
      return nobj2;
    }
    const nobj = {};
    for (const key in obj2) {
      const value = obj2[key];
      if (filter(value))
        nobj[key] = handleObject(value, depth + 1);
    }
    return nobj;
  }
  return handleObject(obj, 0);
}

export {
  objectMap,
  objectMergeDisposable,
  objectIsEmpty,
  objectPick,
  objectInclusivePick,
  objectOmit,
  objectPlain
};
//# sourceMappingURL=chunk-PE4JT3J7.js.map